(function() {

const Cc = Components.classes, Ci = Components.interfaces;
let prefService = Cc['@mozilla.org/preferences-service;1'].getService(Ci.nsIPrefService);
let prefBranch = prefService.getBranch('extensions.stratumbuddy.').QueryInterface(Ci.nsIPrefBranch2);
let prefGeneral = prefService.QueryInterface(Ci.nsIPrefBranch2);
let ssService = Cc['@mozilla.org/content/style-sheet-service;1'].getService(Ci.nsIStyleSheetService);
let ioService = Cc['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService);
let sstoreService = Cc["@mozilla.org/browser/sessionstore;1"].getService(Ci.nsISessionStore);

const BaseURI = ioService.newURI('chrome://StratumBuddy/skin/', null, null);

function enableStyleSheet(sheet) {
  if (!ssService.sheetRegistered(sheet, ssService.USER_SHEET))
    ssService.loadAndRegisterSheet(sheet, ssService.USER_SHEET);
}

function disableStyleSheet(sheet) {
  if (ssService.sheetRegistered(sheet, ssService.USER_SHEET))
    ssService.unregisterSheet(sheet, ssService.USER_SHEET);
}

function loadStyleSheet(url, base) {
  var sheet = ioService.newURI(url, null, base);
  enableStyleSheet(sheet);
  return sheet;
}

function getEmbedCSSURL(css) {
  return 'data:text/css;charset=utf-8,' + encodeURI(css);
}

const Dwm = {

  _dwm: null,
  _navBarBox: null,
  _toolBarBox: null,

  onGlassChange: {
    handleEvent: function(evt) {
      if (Dwm._dwm.GlassEnabled) {
        document.documentElement.setAttribute('stratum-glasser', true);
        gBrowser.addEventListener('resize', Dwm.onWindowResized, false);
        Dwm.onWindowResized.handleEvent();
      } else {
        document.documentElement.removeAttribute('stratum-glasser');
        gBrowser.removeEventListener('resize', Dwm.onWindowResized, false);
      }
    }
  },

  onWindowResized: {
    disabled: false,
    handleEvent: function(evt) {
      if (window.fullScreen) {
        if (!this.disabled) {
          document.documentElement.removeAttribute('stratum-glasser');
          document.documentElement.setAttribute('sb-fullscreen', true);
          Dwm._dwm.ExtendFrameIntoWindow(0, 0, 0, 0);
          this.disabled = true;
        }
      } else {
        document.documentElement.setAttribute('stratum-glasser', true);
        document.documentElement.removeAttribute('sb-fullscreen');
        Dwm._dwm.ExtendFrameIntoWindow(Dwm._navBarBox.height + Dwm._toolBarBox.height, 0, 0, 0);
        this.disabled = false;
      }
    }
  },

  enable: function(ss) {
    if (!this._dwm) {
      this._dwm = Cc['@sixxgate.com/DwmCalls/CDwmCalls;1'].createInstance().QueryInterface(Ci.IDwmCalls);
      this._dwm.AttachToWindow(window);
      this._navBarBox = document.getElementById('nav-bar').boxObject;
      this._toolBarBox = document.getElementById('toolbar-menubar').boxObject;
    }
    this._dwm.OnGlassChange = this.onGlassChange;
    this.onGlassChange.handleEvent();
  },

  disable: function() {
    this._dwm.OnGlassChange = null;
    gBrowser.removeEventListener('resize', this.onWindowResized, false);
    this.onWindowResized.disable = false;
    this._dwm.ExtendFrameIntoWindow(0, 0, 0, 0);
  }
  
};

const BackgroundColor = {
  _ref: null,
  change: function(newval) {
    if (this._ref) disableStyleSheet(this._ref);
    this._ref = newval ? loadStyleSheet(getEmbedCSSURL('\
window, dialog, prefwindow, page{\
  background:' + newval + '!important;\
}\
    '), null) : null;
  }
};

const Prefs = {

  _prefs: {
    'isvista': {
      true: {
        css: 'Vista.css',
        binding: Dwm
      }
    },
     'enableOverride': {
      true: {
        css: 'override.css',
      }
    },
    'BackgroundColor': {
      binding: BackgroundColor
    },
     'showBookmarkPage': {
      att: 'sb-showBookmarkPage',
    },
     'showOrganizeBookmarks': {
      att: 'sb-showOrganizeBookmarks',
    },
     'tabLayout': {
      att: 'sb-tabLayout',
    },
     'appButtonColor': {
      att: 'sb-appButtonColor',
    },
     'usefulbarLayout': {
      att: 'sb-usefulbarLayout',
    },
     'usefulbar-Bookmarks': {
      att: 'sb-usefulbar-Bookmarks',
    },
     'usefulbar-History': {
      att: 'sb-usefulbar-History',
    },
     'usefulbar-TabHistory': {
      att: 'sb-usefulbar-TabHistory',
    },
     'usefulbar-Downloads': {
      att: 'sb-usefulbar-Downloads',
    },
     'usefulbar-Home': {
      att: 'sb-usefulbar-Home',
    }
  },

  getPrefValue: function(key) {
    switch (prefBranch.getPrefType(key)) {
      case prefBranch.PREF_INT:
        return prefBranch.getIntPref(key);
      case prefBranch.PREF_BOOL:
        return prefBranch.getBoolPref(key);
      case prefBranch.PREF_STRING:
        return prefBranch.getCharPref(key);
    }
    return null;
  },

  enableNew: function(pref, val) {
    if ('att' in pref)
      document.documentElement.setAttribute(pref.att, val);
    if ('binding' in pref)
      pref.binding.change(val, pref.last);
    pref.last = val;
    if (!(val in pref))
      return;
    var valOpt = pref[val];
    if ('css' in valOpt)
      pref.ref = loadStyleSheet(valOpt.css, BaseURI);
    if ('binding' in valOpt)
      valOpt.binding.enable(pref.ref, val);
  },

  disableLast: function(pref) {
    if (!('last' in pref) || !(pref.last in pref))
      return;
    if (pref.ref) {
      disableStyleSheet(pref.ref);
      delete pref.ref;
    }
    if ('binding' in pref[pref.last])
      pref[pref.last].binding.disable();
  },

  onPrefChange : {
    observe: function(subject, topic, key) {
      if (!(key in Prefs._prefs))
        return;
      var pref = Prefs._prefs[key];
      var val = Prefs.getPrefValue(key);
      if (pref.last == val)
        return;
      Prefs.disableLast(pref);
      if (val != null)
        Prefs.enableNew(pref, val);
    }
  },

  enable: function() {
    for (var key in this._prefs) {
      var val = this.getPrefValue(key);
      if (val != null)
        this.enableNew(this._prefs[key], val);
    }
    prefBranch.addObserver('', this.onPrefChange, false);
  },

  disable: function() {
    for each (var pref in this._prefs)
      this.disableLast(pref);
    prefBranch.removeObserver('', this.onPrefChange);
  }

};

function init() {
  window.removeEventListener('load', init, false);
  prefBranch.setBoolPref('isvista', navigator.oscpu.indexOf('NT 6') > -1);
  Prefs.enable();
  //force our layout for now
  //AddToolbarButtons();
  //move bookmark star to start of urlbar
  document.getElementById('urlbar').insertBefore((document.getElementById('star-button')), null);
  //put back forward in front of urlbar
  var sbcontainer = document.getElementById('unified-back-forward-urlbar');
  sbcontainer.parentNode.insertBefore(sbcontainer,sbcontainer.parentNode.firstChild);
  //Fix 'Bookmark Sidebar' label bug
  var strings = document.getElementById("string-bundle");
  var booklabel = document.getElementById("bookmarkside-item");
  if (booklabel) booklabel.setAttribute("label", strings.getString('bookmarkSidebar'));
  // Remove localized "Done" text..hopefully
  var donetext = gNavigatorBundle.getString("nv_done")
  loadStyleSheet(getEmbedCSSURL('\
.statusbarpanel-text[value="'+ donetext +'"] {\
  display: none !important;\
}\
  '));
  setTimeout(historyListener, 100);
}

window.StratumBuddy = {
  setMainMenu: function(evt) {
    var pbs = Cc["@mozilla.org/privatebrowsing;1"].getService(Ci.nsIPrivateBrowsingService);
    var inPrivateBrowsingMode = pbs.privateBrowsingEnabled;
    var ptoolbar = document.getElementById("PersonalToolbar");
    var mtoolbar = document.getElementById("toolbar-menubar");
    document.getElementById('privatebrowse-item').setAttribute('checked', inPrivateBrowsingMode);
    document.getElementById('personaltoolbar-item').setAttribute('checked', ptoolbar.getAttribute("collapsed") != "true");
    document.getElementById('menubar-item').setAttribute('checked', mtoolbar.getAttribute("autohide") != "true");
  },
  PopulateHistory: function() {
    var undoPopup = document.getElementById("tab-menu-button-popup");
    var historybutton = document.getElementById("tab-menu-button");
    var undoItems = eval("(" + sstoreService.getClosedTabData(window) + ")");
    while (undoPopup.hasChildNodes())
      undoPopup.removeChild(undoPopup.firstChild);
    for (var i = 0; i < undoItems.length; i++) {
      var m = document.createElement("menuitem");
      m.setAttribute("label", undoItems[i].title);
      if (undoItems[i].image) {
        let iconURL = undoItems[i].image;
        if (/^https?:/.test(iconURL))
          iconURL = "moz-anno:favicon:" + iconURL;
        m.setAttribute("image", iconURL);
      }
      m.setAttribute("class", "menuitem-iconic bookmark-item");
      m.setAttribute("value", i);
      m.setAttribute("oncommand", "undoCloseTab(" + i + ");");
      if (i == 0)
        m.setAttribute("key", "key_undoCloseTab");
      undoPopup.appendChild(m);
    }
  },
  onMenuCommandToggle: function(evt) {
    var el = evt.target;
    var toolbar = document.getElementById(el.value);
    if (el.value == 'PersonalToolbar') {
      toolbar.setAttribute('collapsed', !el.hasAttribute('checked'));
      document.persist(el.value, 'collapsed');
    } else if (el.value == 'toolbar-menubar'){
      toolbar.setAttribute('autohide', !el.hasAttribute('checked'));
      document.persist(el.value, 'autohide');
    }
  },
    calculateColors: function() {
		var rgb = "rgb(" + (document.getElementById('a1').value) + "," + (document.getElementById('a2').value) + "," + (document.getElementById('a3').value) + ")"
		prefBranch.setCharPref('BackgroundColor',rgb)
  },
    showItemList: function () {
    var itemList = document.getElementById("itemList");
    var expanderButton = document.getElementById("detailsExpander");
    if (itemList.collapsed) {
      expanderButton.className = "expander-up";
      itemList.setAttribute("collapsed", "false");
    }
  },
  hideItemList: function () {
    var itemList = document.getElementById("itemList");
    var expanderButton = document.getElementById("detailsExpander");
    if (!itemList.collapsed) {
      expanderButton.className = "expander-down";
      itemList.setAttribute("collapsed", "true");
    }
  },
  toggleItemList: function ()
  {
    var itemList = document.getElementById("itemList");

    if (itemList.collapsed)
      this.showItemList();
    else
      this.hideItemList();
  }
};

function AddToolbarButtons() {
  var firefoxnav = document.getElementById("nav-bar");
  var set = "urlbar-container,sb-pagetools-container";
  firefoxnav.setAttribute("currentset", set);
  firefoxnav.currentSet = set;
  document.persist("nav-bar", "currentset");
  BrowserToolboxCustomizeDone(true);
}

function historyListener() {
  checkHistory();
  gBrowser.mTabContainer.addEventListener("TabOpen", checkHistory, false);
  gBrowser.mTabContainer.addEventListener("TabClose", checkHistory, false);
}

function checkHistory() {
  var history = document.getElementById("tab-menu-button");
  if (history) history.disabled = sstoreService.getClosedTabCount(window) == 0;
}

window.addEventListener('load', init, false);
})();